FUNCTION ValidTime
* Syntax ...: logical = ValidTime(<time string>)
* Author ...: Tom Rettig
* Date .....: 11/01/85, 09/01/86
*
PARAMETERS timestring
RETURN VAL(timestring)            < 24 .AND.;
       VAL(SUBSTR(timestring, 4)) < 60 .AND.;
       VAL(SUBSTR(timestring, 7)) < 60

***
*   AMPM()
*   Tom Rettig
*   11/01/85, 09/01/86
*
*   Returns an 11 byte character string with the time in
*       a 12-hour am/pm format.
*
*   time_ampm = AMPM(time_24)
*
*       time_ampm   -   character string.
*       time_24     -   character string.
*
*   Placed in the public domain by Tom Rettig Associates.
*
*

FUNCTION AMPM

PARAMETERS cl_time

RETURN IF(        VAL(cl_time)<12, cl_time + " am",;
           IF(    VAL(cl_time)=12, cl_time + " pm",;
              STR(VAL(cl_time)-12,2) + SUBSTR(cl_time,3) + " pm" ) )
** eofunc AMPM()


****
*
*       DAYS()
*
***
FUNCTION DAYS
* Syntax: DAYS( <seconds> )
* Return: Integer number of days from numeric seconds
* Note..: The remainder under 24 hours is returned by the TSTRING() function.
*
PARAMETERS cl_secs
RETURN INT( cl_secs / 86400 )

****
* Syntax: ELAPTIME( <start time>, <end time> )
* Return: A time string showing the difference between start and end times
* Note..: If start time is greater than end time, this algorithm assumes
*         that the day changed at midnight.  Only for timings under 24 hours.
*         86400 is the number of seconds in 24 hours.
***

FUNCTION ELAPTIME
PARAMETERS cl_start, cl_end
RETURN TSTRING( IF(cl_end<cl_start,86400,0)+SECS(cl_end)-SECS(cl_start) )

****
* Syntax: SECS( <time string> )
* Return: Numeric seconds as a quantity of the time string
* Note..: Seconds in time period
*         -------    -----------
*              60    1 minute
*            3600    1 hour
*           86400    1 day
*
***
FUNCTION SECS
PARAMETERS cl_time
RETURN VAL(       cl_time   ) * 3600 +;
       VAL(SUBSTR(cl_time,4)) *   60 +;
       VAL(SUBSTR(cl_time,7))

****
* Syntax: TSTRING( <seconds> )
* Return: A 24-hour time string from numeric seconds
* Note..: Time quantities over 24 hours are returned by the DAYS() function.
*
***
FUNCTION TSTRING
PARAMETERS cl_secs
RETURN STRZERO( INT(MOD(cl_secs/3600, 24)), 2, 0 ) +':'+;
       STRZERO( INT(MOD(cl_secs/  60, 60)), 2, 0 ) +':'+;
       STRZERO( INT(MOD(cl_secs     , 60)), 2, 0 )


****
*
* Syntax: DBF()
* Return: The alias of the currently selected database.
* Note..: Supposed to return the name of the currently selected database file.
*
***
FUNCTION DBF
RETURN ALIAS()

****
* Syntax: FKLABEL( <expN> )
* Return: The name of the <expN>th programmable function key
***
FUNCTION FKLABEL
PARAMETERS cl_1
RETURN IF(cl_1<=40 .AND. cl_1 > 0, "F"+LTRIM(STR(cl_1)), [])


****
*
* Syntax: FKMAX()
* Return: The maximum number of programmable function keys on the computer
* Note..: This is specific to IBM PC/XT/AT and clones.
*
***
FUNCTION FKMAX
RETURN 40          && IBM specific


****
* Syntax OS()
* Return: The name of the operating system
*
***
FUNCTION OS
RETURN "MS/PC-DOS"    && DOS specific, MicroSoft or IBM


****
* Syntax: READKEY()
* Return: A number representing the key pressed to exit from full-screen mode
* Note..: Differences between dBASE's READKEY() and Clipper's LASTKEY():
*
*         Exit Key:      dBASE:      Clipper:
*         ---------      ------      --------
*         Backspace         0        no exit
*         ^D, ^L            1        no exit
*         Lt arrow          2        no exit
*         Rt arrow          3        no exit
*         Up arrow          4        no exit
*         Dn arrow          5        no exit
*         PgUp              6          18
*         PgDn              7           3
*         Esc, ^Q          12          27 (Esc only)
*         ^End, ^W         14          23 (^W only)
*         type past end    15        ascii of last char typed
*         Enter            15          13
*         ^Home            33        no exit
*         ^PgUp            34        no exit
*         ^PgDn            35        no exit
*         F1               36        no exit
*
*         dBASE III adds 256 to the exit code if the user changed anything.
*         Clipper uses its UPDATED() function to determine if anything changed.
*
***
FUNCTION READKEY
DO CASE
   CASE LASTKEY() = 18                           && PgUp
      RETURN  6 + IF(UPDATED(),256,0)
   CASE LASTKEY() =  3                           && PgDn
      RETURN  7 + IF(UPDATED(),256,0)
   CASE LASTKEY() = 27                           && Esc
      RETURN 12 + IF(UPDATED(),256,0)
   CASE LASTKEY() = 23                           && ^W
      RETURN 14 + IF(UPDATED(),256,0)
   CASE LASTKEY() = 13                           && Enter
      RETURN 15 + IF(UPDATED(),256,0)
   CASE LASTKEY() = 31                           && ^PgUp
      RETURN 34 + IF(UPDATED(),256,0)
   CASE LASTKEY() = 30                           && ^PgDn
      RETURN 35 + IF(UPDATED(),256,0)
   CASE LASTKEY() >= 32                          && type past end
      RETURN 15 + IF(UPDATED(),256,0)
   OTHERWISE
      RETURN 0
ENDCASE

****
* Syntax: VERSION()
* Return: The name of current dBASE III or Clipper version
* Note..: Remember to change this when you update your Clipper version.
***
FUNCTION VERSION
RETURN "Clipper, Summer '87"

****
*   STRZERO()
*   Tom Rettig
*   11/01/85, 09/01/86
*
*   The STR() of <expN> with leading zeros instead of blanks.
*
*   string = STRZERO( <expN>, [<length> [,<decimals>]] )
*
*       string      -   character string.
*       expN        -   numeric.
*       length      -   numeric.
*       decimals    -   numeric.
*
*   Placed in the public domain by Tom Rettig Associates.
*
FUNCTION STRZERO
PARAMETERS cl_num, cl_len, cl_dec
PRIVATE cl_str
DO CASE
   CASE PCOUNT()=3
      cl_str = STR(cl_num,cl_len,cl_dec)
   CASE PCOUNT()=2
      cl_str = STR(cl_num,cl_len)
   OTHERWISE
      cl_str = STR(cl_num)
ENDCASE
IF "-" $ cl_str    && negative number
   * Move the minus sign in front of zeros.
   RETURN "-" + REPLICATE( "0", LEN(cl_str)-LEN(LTRIM(cl_str)) ) + SUBSTR( cl_str, AT("-",cl_str)+1 )
ELSE   && positive number
   RETURN REPLICATE( "0", LEN(cl_str)-LEN(LTRIM(cl_str)) ) + LTRIM(cl_str)
ENDIF

*
*
** eofunc STRZERO()
****
* Syntax: MOD( <expN1>, <expN2> )
* Return: The remainder of <expN1> divided by <expN2>
* Note..: Difference between the dBASE modulus function and the Clipper
*         modulus operator is indicated by an arrow <-->:
*
*         Clipper operator:        dBASE function:
*         -----------------        -----------------
*          3 %  3 ::=  0.00        MOD( 3, 3) ::=  0
*          3 %  2 ::=  1.00        MOD( 3, 2) ::=  1
*          3 %  1 ::=  0.00        MOD( 3, 1) ::=  0
*          3 %  0 ::=  0.00  <-->  MOD( 3, 0) ::=  3
*          3 % -1 ::=  0.00        MOD( 3,-1) ::=  0
*          3 % -2 ::=  1.00  <-->  MOD( 3,-2) ::= -1
*          3 % -3 ::=  0.00        MOD( 3,-3) ::=  0
*
*         -3 %  3 ::=  0.00        MOD(-3, 3) ::=  0
*         -3 %  2 ::= -1.00  <-->  MOD(-3, 2) ::=  1
*         -3 %  1 ::=  0.00        MOD(-3, 1) ::=  0
*         -3 %  0 ::=  0.00  <-->  MOD(-3, 0) ::= -3
*         -3 % -1 ::=  0.00        MOD(-3,-1) ::=  0
*         -3 % -2 ::= -1.00        MOD(-3,-2) ::= -1
*         -3 % -3 ::=  0.00        MOD(-3,-3) ::=  0
*
*          3 %  3 ::=  0.00        MOD( 3, 3) ::=  0
*          2 %  3 ::=  2.00        MOD( 2, 3) ::=  2
*          1 %  3 ::=  1.00        MOD( 1, 3) ::=  1
*          0 %  3 ::=  0.00        MOD( 0, 3) ::=  0
*         -1 %  3 ::= -1.00  <-->  MOD(-1, 3) ::=  2
*         -2 %  3 ::= -2.00  <-->  MOD(-2, 3) ::=  1
*         -3 %  3 ::=  0.00        MOD(-3, 3) ::=  0
*
*          3 % -3 ::=  0.00        MOD( 3,-3) ::=  0
*          2 % -3 ::=  2.00  <-->  MOD( 2,-3) ::= -1
*          1 % -3 ::=  1.00  <-->  MOD( 1,-3) ::= -2
*          0 % -3 ::=  0.00        MOD( 0,-3) ::=  0
*         -1 % -3 ::= -1.00        MOD(-1,-3) ::= -1
*         -2 % -3 ::= -2.00        MOD(-2,-3) ::= -2
*         -3 % -3 ::=  0.00        MOD(-3,-3) ::=  0
*
***
FUNCTION MOD
PARAMETERS cl_num, cl_base
PRIVATE cl_result
cl_result = cl_num % cl_base
RETURN IF( cl_base = 0, cl_num,;
           IF(cl_result * cl_base < 0, cl_result + cl_base, cl_result) )

